@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2022 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2015-2024 Damon Lynch <damonlynch@gmail.com>
 # Copyright (c) 2012-2014 Alexander Turkin
 
 # This file is part of Rapid Photo Downloader.
@@ -36,83 +36,82 @@ Copyright notice from QtWaitingSpinner source:
 """
 
 __author__ = "Damon Lynch"
-__copyright__ = "Copyright 2015-2022, Damon Lynch"
+__copyright__ = "Copyright 2015-2024, Damon Lynch"
 
-import math
-from collections import namedtuple, defaultdict
-from typing import Optional, Dict, List, Set
 import logging
+import math
+from collections import defaultdict, namedtuple
 
 from PyQt5.QtCore import (
+    QAbstractItemModel,
+    QAbstractListModel,
+    QEvent,
     QModelIndex,
-    QSize,
-    Qt,
+    QObject,
     QPoint,
     QPointF,
     QRect,
     QRectF,
-    QEvent,
-    QAbstractItemModel,
-    QAbstractListModel,
-    pyqtSlot,
-    pyqtSignal,
+    QSize,
+    Qt,
     QTimer,
-    QObject,
-)
-from PyQt5.QtWidgets import (
-    QStyledItemDelegate,
-    QStyleOptionViewItem,
-    QApplication,
-    QStyle,
-    QStyleOptionButton,
-    QAbstractItemView,
-    QMenu,
-    QWidget,
+    pyqtSignal,
+    pyqtSlot,
 )
 from PyQt5.QtGui import (
-    QPainter,
-    QFontMetrics,
-    QFont,
-    QColor,
-    QLinearGradient,
     QBrush,
-    QPalette,
-    QPixmap,
-    QPaintEvent,
+    QColor,
+    QFont,
+    QFontMetrics,
     QGuiApplication,
-    QPen,
     QIcon,
+    QLinearGradient,
+    QPainter,
+    QPaintEvent,
+    QPalette,
+    QPen,
+    QPixmap,
+)
+from PyQt5.QtWidgets import (
+    QAbstractItemView,
+    QApplication,
+    QMenu,
+    QStyle,
+    QStyledItemDelegate,
+    QStyleOptionButton,
+    QStyleOptionViewItem,
+    QWidget,
 )
 
-from raphodo.ui.viewutils import (
-    RowTracker,
-    ListViewFlexiFrame,
-    device_name_highlight_color,
-    standard_font_size,
-    scaledIcon,
-    darkModePixmap,
-    is_dark_mode,
-)
 from raphodo.constants import (
-    DeviceState,
-    FileType,
-    CustomColors,
-    DeviceType,
-    Roles,
-    EmptyViewHeight,
-    ViewRowType,
     Checked_Status,
+    CustomColors,
     DeviceDisplayPadding,
     DeviceShadingIntensity,
+    DeviceState,
+    DeviceType,
     DisplayingFilesOfType,
+    DownloadFailure,
     DownloadStatus,
     DownloadWarning,
-    DownloadFailure,
+    EmptyViewHeight,
+    FileType,
+    Roles,
+    ViewRowType,
 )
 from raphodo.devices import Device
-from raphodo.utilities import thousands, format_size_for_user
-from raphodo.storage.storage import StorageSpace
 from raphodo.rpdfile import make_key
+from raphodo.storage.storage import StorageSpace  # noqa: F401
+from raphodo.ui.viewutils import (
+    ListViewFlexiFrame,
+    RowTracker,
+    darkModePixmap,
+    device_name_highlight_color,
+    is_dark_mode,
+    scaledIcon,
+    standard_font_size,
+)
+from raphodo.utilities import format_size_for_user, thousands
 
 
 def icon_size() -> int:
@@ -142,25 +141,25 @@ class DeviceModel(QAbstractListModel):
         self.rapidApp = parent
         self.device_display_type = device_display_type
         # scan_id: Device
-        self.devices = {}  # type: Dict[int, Device]
+        self.devices = {}  # type: dict[int, Device]
         # scan_id: DeviceState
-        self.spinner_state = {}  # type: Dict[int, DeviceState]
+        self.spinner_state = {}  # type: dict[int, DeviceState]
         # scan_id: bool
-        self.checked = defaultdict(lambda: Qt.Checked)  # type: Dict[int, Qt.CheckState]
-        self.icons = {}  # type: Dict[int, QPixmap]
+        self.checked = defaultdict(lambda: Qt.Checked)  # type: dict[int, Qt.CheckState]
+        self.icons = {}  # type: dict[int, QPixmap]
         self.rows = RowTracker()  # type: RowTracker
         self.row_id_counter = 0  # type: int
-        self.row_id_to_scan_id = dict()  # type: Dict[int, int]
-        self.scan_id_to_row_ids = defaultdict(list)  # type: Dict[int, List[int]]
-        self.storage = dict()  # type: Dict[int, Optional[StorageSpace]]
-        self.headers = set()  # type: Set[int]
+        self.row_id_to_scan_id = dict()  # type: dict[int, int]
+        self.scan_id_to_row_ids = defaultdict(list)  # type: dict[int, list[int]]
+        self.storage = dict()  # type: dict[int, StorageSpace | None]
+        self.headers = set()  # type: set[int]
 
         self.icon_size = icon_size()
 
-        self.row_ids_active = []  # type: List[int]
+        self.row_ids_active = []  # type: list[int]
 
         # scan_id: 0.0-1.0
-        self.percent_complete = defaultdict(float)  # type: Dict[int, float]
+        self.percent_complete = defaultdict(float)  # type: dict[int, float]
 
         self._rotation_position = 0  # type: int
         self._timer = QTimer(self)
@@ -330,7 +329,6 @@ class DeviceModel(QAbstractListModel):
         self.dataChanged.emit(self.index(row, 0), self.index(row, 0))
 
     def data(self, index: QModelIndex, role=Qt.DisplayRole):
-
         if not index.isValid():
             return None
 
@@ -385,9 +383,11 @@ class DeviceModel(QAbstractListModel):
 
         if role == Qt.CheckStateRole:
             # In theory, update checkbox immediately, as selecting a very large number
-            # of thumbnails can take time. However the code is probably wrong, as it
+            # of thumbnails can take time. However, the code is probably wrong, as it
             # doesn't work:
-            # self.setCheckedValue(checked=value, scan_id=scan_id, row=row, log_state_change=False)
+            # self.setCheckedValue(
+            #   checked=value, scan_id=scan_id, row=row, log_state_change=False
+            # )
             # QApplication.instance().processEvents()
             self.rapidApp.thumbnailModel.checkAll(value, scan_id=scan_id)
             return True
@@ -408,20 +408,18 @@ class DeviceModel(QAbstractListModel):
             logging.debug(
                 "Spinner states: %s",
                 ", ".join(
-                    "%s: %s"
-                    % (
-                        self.devices[scan_id].display_name,
-                        self.spinner_state[scan_id].name,
+                    (
+                        f"{self.devices[scan_id].display_name}: "
+                        f"{self.spinner_state[scan_id].name}"
                     )
                     for scan_id in self.spinner_state
                 ),
             )
             logging.debug(
                 ", ".join(
-                    "%s: %s"
-                    % (
-                        self.devices[scan_id].display_name,
-                        Checked_Status[self.checked[scan_id]],
+                    (
+                        f"{self.devices[scan_id].display_name}: "
+                        f"{Checked_Status[self.checked[scan_id]]}"
                     )
                     for scan_id in self.checked
                 )
@@ -431,8 +429,8 @@ class DeviceModel(QAbstractListModel):
         self,
         checked: Qt.CheckState,
         scan_id: int,
-        row: Optional[int] = None,
-        log_state_change: Optional[bool] = True,
+        row: int | None = None,
+        log_state_change: bool | None = True,
     ) -> None:
         logging.debug(
             "Setting %s checkbox to %s",
@@ -476,8 +474,8 @@ class DeviceView(ListViewFlexiFrame):
     def __init__(
         self,
         rapidApp,
-        frame_enabled: Optional[bool] = True,
-        parent: Optional[QWidget] = None,
+        frame_enabled: bool | None = True,
+        parent: QWidget | None = None,
     ) -> None:
         super().__init__(frame_enabled=frame_enabled, parent=parent)
         self.rapidApp = rapidApp
@@ -590,7 +588,7 @@ class DeviceComponent(QObject):
 
     widthChanged = pyqtSignal(int)
 
-    def __init__(self, parent: Optional[QObject] = None) -> None:
+    def __init__(self, parent: QObject | None = None) -> None:
         super().__init__(parent=parent)
         style = QApplication.style()
         self.frame_width = style.pixelMetric(QStyle.PM_DefaultFrameWidth)
@@ -635,7 +633,7 @@ class DeviceComponent(QObject):
             sample_comp3
         ).width()
 
-        # Height of the details about the storage e.g. number of photos
+        # Height of the details about the storage e.g., number of photos,
         # videos, etc.
         self.details_height = self.deviceFontMetrics.height() * 2 + 2
         self.details_vertical_bar_width = 10
@@ -701,7 +699,7 @@ class DeviceDisplay(QObject):
     shading_intensity = DeviceShadingIntensity
     widthChanged = pyqtSignal(int)
 
-    def __init__(self, parent: QObject, menuButtonIcon: Optional[QIcon] = None) -> None:
+    def __init__(self, parent: QObject, menuButtonIcon: QIcon | None = None) -> None:
         super().__init__(parent)
         self.menuButtonIcon = menuButtonIcon
 
@@ -746,7 +744,7 @@ class DeviceDisplay(QObject):
         Render the header portion, which contains the device / folder name, icon, and
         for download sources, a spinner or checkbox.
 
-        If needed, draw a pixmap for for a drop-down menu.
+        If needed, draw a pixmap for a drop-down menu.
         """
 
         painter.setRenderHint(QPainter.Antialiasing, True)
@@ -868,7 +866,7 @@ class DeviceDisplay(QObject):
         # ==========================================================================
 
         # Devices may not have photos or videos
-        # Fill in storage bar with size of photos
+        # Fill in storage bar with the size of the photos
         if comp1_file_size_sum and d.bytes_total:
             photos_g_rect = QRectF(
                 photos_g_x, g_y, photos_g_width, self.dc.storage_use_bar_height
@@ -892,7 +890,7 @@ class DeviceDisplay(QObject):
         else:
             videos_g_width = 0
 
-        # Fill in storage bar with size of other files
+        # Fill in the storage bar with size of other files
         if comp3_file_size_sum and d.bytes_total:
             other_g_width = comp3_file_size_sum / d.bytes_total * width
             other_g_x = videos_g_x + videos_g_width
@@ -975,7 +973,6 @@ class DeviceDisplay(QObject):
         # ===================
 
         if not skip_comp1:
-
             # Gradient
             photos_g2_rect = QRectF(
                 photos_g2_x,
@@ -1160,10 +1157,9 @@ class AdvancedDeviceDisplay(DeviceDisplay):
         device_state: DeviceState,
         rotation: int,
         checked: bool,
-        download_statuses: Set[DownloadStatus],
+        download_statuses: set[DownloadStatus],
         percent_complete: float,
     ) -> None:
-
         standard_pen_color = painter.pen().color()
 
         super().paint_header(
@@ -1171,8 +1167,8 @@ class AdvancedDeviceDisplay(DeviceDisplay):
         )
 
         if device_state == DeviceState.finished:
-            # indicate that no more files can be downloaded from the device, and if there
-            # were any errors or warnings
+            # indicate that no more files can be downloaded from the device, and if
+            # there were any errors or warnings
             size = QSize(self.downloaded_icon_size, self.downloaded_icon_size)
             if download_statuses & DownloadFailure:
                 pixmap = self.downloadedErrorIcon.pixmap(size)
@@ -1185,7 +1181,6 @@ class AdvancedDeviceDisplay(DeviceDisplay):
             )
 
         elif device_state not in (DeviceState.scanning, DeviceState.downloading):
-
             checkboxStyleOption = QStyleOptionButton()
             if checked == Qt.Checked:
                 checkboxStyleOption.state |= QStyle.State_On
@@ -1244,7 +1239,6 @@ class AdvancedDeviceDisplay(DeviceDisplay):
             painter.setPen(standard_pen_color)
 
     def paint_alternate(self, painter: QPainter, x: int, y: int, text: str) -> None:
-
         standard_pen_color = painter.pen().color()
 
         painter.setPen(standard_pen_color)
@@ -1288,7 +1282,6 @@ class AdvancedDeviceDisplay(DeviceDisplay):
 
 
 class DeviceDelegate(QStyledItemDelegate):
-
     padding = DeviceDisplayPadding
 
     probing_text = _("Probing device...")
@@ -1315,8 +1308,8 @@ class DeviceDelegate(QStyledItemDelegate):
         self.blacklistDeviceAct.triggered.connect(self.blacklistDevice)
         self.rescanDeviceAct = self.contextMenu.addAction(_("Rescan"))
         self.rescanDeviceAct.triggered.connect(self.rescanDevice)
-        # store the index in which the user right clicked
-        self.clickedIndex = None  # type: Optional[QModelIndex]
+        # store the index in which the user right-clicked
+        self.clickedIndex = None  # type: QModelIndex | None
 
     @pyqtSlot()
     def ignoreDevice(self) -> None:
@@ -1359,9 +1352,7 @@ class DeviceDelegate(QStyledItemDelegate):
                 Roles.device_details
             )
             if device_state == DeviceState.finished:
-                download_statuses = index.data(
-                    Roles.download_statuses
-                )  # type: Set[DownloadStatus]
+                download_statuses = index.data(Roles.download_statuses)  # type: set[DownloadStatus]
             else:
                 download_statuses = set()
 
@@ -1387,12 +1378,9 @@ class DeviceDelegate(QStyledItemDelegate):
         else:
             assert view_type == ViewRowType.content
 
-            device, storage_space = index.data(
-                Roles.storage
-            )  # type: Device, StorageSpace
+            device, storage_space = index.data(Roles.storage)  # type: Device, StorageSpace
 
             if storage_space is not None:
-
                 if device.device_type == DeviceType.camera:
                     photo_key = make_key(FileType.photo, storage_space.path)
                     video_key = make_key(FileType.video, storage_space.path)
@@ -1430,9 +1418,7 @@ class DeviceDelegate(QStyledItemDelegate):
                         storage_space.bytes_total, no_decimals=0
                     )
                     bytes_used = storage_space.bytes_total - storage_space.bytes_free
-                    percent_used = "{0:.0%}".format(
-                        bytes_used / storage_space.bytes_total
-                    )
+                    percent_used = f"{bytes_used / storage_space.bytes_total:.0%}"
                     # Translators: percentage full e.g. 75% full
                     percent_used = _("%s full") % percent_used
                     bytes_total = storage_space.bytes_total
@@ -1503,7 +1489,7 @@ class DeviceDelegate(QStyledItemDelegate):
         """
         Change the data in the model and the state of the checkbox
         if the user presses the left mousebutton or presses
-        Key_Space or Key_Select and this cell is editable. Otherwise do nothing.
+        Key_Space or Key_Select and this cell is editable. Otherwise, do nothing.
         """
 
         if (
@@ -1511,7 +1497,7 @@ class DeviceDelegate(QStyledItemDelegate):
             or event.type() == QEvent.MouseButtonDblClick
         ):
             if event.button() == Qt.RightButton:
-                # Disable ignore and blacklist menus if the device is a This Computer
+                # Disable ignore and blacklist menus if the device is a "This Computer"
                 # path
 
                 self.clickedIndex = index
@@ -1552,7 +1538,7 @@ class DeviceDelegate(QStyledItemDelegate):
         return True
 
     def setModelData(
-        self, editor: Optional[QWidget], model: QAbstractItemModel, index: QModelIndex
+        self, editor: QWidget | None, model: QAbstractItemModel, index: QModelIndex
     ) -> None:
         newValue = not (index.model().data(index, Qt.CheckStateRole))
         model.setData(index, newValue, Qt.CheckStateRole)
